#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/extensions/sync.h>

static void usage()
{
    fprintf(stderr, "usage:\n");
    fprintf(stderr, "-a <alarm> | -c <counter> | -s <system counter>\n");
    fprintf(stderr, "-i <increment>\n");
    fprintf(stderr, "-l <limit>\n");
    exit(1);
}

int main(argc, argv)
int argc;
char **argv;
{
    Display *dpy;
    unsigned long sync_event, sync_error;
    int major_val, minor_val, count;
    unsigned long n_counters;
    SystemCounterList sys_counter_list;
    Counter counter = 0;
    char *counter_name = "SERVERTIME";
    Alarm alarm=0;
    unsigned long values_mask;
    XSyncAlarmAttributes values;
    unsigned long increment = 0;
    unsigned long limit = 0;

    while (argc>1) {
	if (argv[1][0] == '-') {
	    switch (argv[1][1]) {
	    case 'c':
		if (argc==2) usage();
		counter = atoi(argv[2]);
		sprintf(counter_name, "%d", counter);
		argv += 2;
		argc -= 2;
		break;
	    case 'a':
		if (argc==2) usage();
		alarm = atoi(argv[2]);
		argv += 2;
		argc -= 2;
		break;
	    case 's':
		if (argc==2) usage();
		counter_name = argv[2];
		argv += 2;
		argc -= 2;
		break;
	    case 'i':
		if (argc==2) usage();
		increment = atoi(argv[2]);
		argv += 2;
		argc -= 2;
		break;
	    case 'l':
		if (argc==2) usage();
		limit = atoi(argv[2]);
		argv += 2;
		argc -= 2;
		break;
	    default:
		fprintf(stderr, "Unknown option -%c\n", argv[1][1]);
		usage();
	    }
	} else {
	    usage();
	}
    }

    dpy = XOpenDisplay(NULL);

    if (!dpy) {
	fprintf(stderr, "Failed to open display!\n");
	exit(-1);
    }

    if (!XSyncQueryExtension(dpy, &sync_event, &sync_error)) {
	fprintf(stderr, "Failed to find SYNC extension\n");
	exit(-1);
    }

    XSyncInfo(dpy, &major_val, &minor_val, &sys_counter_list, &n_counters);

    if (alarm == 0) {
	if (counter==0) {
	    for (count=n_counters-1; count>=0; count--) {
		if (strcmp(sys_counter_list[count].name, counter_name)==0) {
		    break;
		}
	    }
	    if (count<0) {
		fprintf(stderr, "Failed to find counter \"%s\"\n",
			counter_name);
		exit(1);
	    }
	    counter = sys_counter_list[count].counter;
	    printf("Found counter '%s', with resolution %.8X%.8X\n",
		   sys_counter_list[count].name,
		   C64High32(sys_counter_list[count].resolution),
		   C64Low32(sys_counter_list[count].resolution));
	} else {
	    printf("Using counter %d\n", counter);
	}
    }

    if (alarm!=0) {
	values.events = 1;
	values_mask = CAEvents;

	XSyncChangeAlarm(dpy, alarm, values_mask, &values);
	printf("Using Alarm %u\n", alarm);
    } else {
	values.counter = counter;

	XSyncQueryCounter(dpy, counter, &values.value);

	if (increment==0) {
	    if (strcmp(counter_name, "SERVERTIME")==0) {
		increment = 1000;
	    } else {
		increment = 1;
	    }
	}
	if (limit==0) {
	    limit = increment*30;
	}

	fprintf(stderr, "increment %d, limit %d\n", increment, limit);

	cvtLongToC64(values.increment, increment);

	cvtLongToC64(values.limit, limit);

	values.limit_type = SyncRelative;

	values_mask = CACounter | CAValue | CAIncrement | CALimit | CALimitType;

	alarm=XSyncCreateAlarm(dpy, values_mask, &values);
	printf("Alarm is %u\n", alarm);
    }

    while (True) {
	XAlarmNotifyEvent event;

	XNextEvent(dpy, (XEvent *)&event);
	if (event.type == sync_event + AlarmNotify) {
	    printf("alarm %8.8X: %8.8X%8.8X, ",
		   event.alarm,
		   C64High32(event.alarm_value),
		   C64Low32(event.alarm_value));
	    printf("counter %8.8X%8.8X ",
		   C64High32(event.counter_value),
		   C64Low32(event.counter_value));
	    printf("time %8.8X, ", event.time);
	    switch( event.state ) {
	    case AlarmActive:
		printf("Active\n");
		break;
	    case AlarmExhausted:
		printf("Exhausted\n");
		break;
	    case AlarmDestroyed:
		printf("Destroyed\n");
		break;
	    default:
		printf("Unknown state %d\n", event.state);
	    }
	    if (argc<=1 && event.state == AlarmExhausted) {
		break;
	    }
	    if (event.state == AlarmDestroyed) {
		break;
	    }
	}
    }
    printf("Your time is up!\n");

    return 0;
}
