#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/extensions/sync.h>

static void usage()
{
    fprintf(stderr, "usage:\n");
    fprintf(stderr, "-c <counter>\n");
    fprintf(stderr, "-i <increment>\n");
    fprintf(stderr, "-l <limit>\n");
    exit(1);
}

int main(argc, argv)
int argc;
char **argv;
{
    Display *dpy;
    unsigned long sync_event, sync_error;
    int major_val, minor_val;
    Counter counter;
    unsigned long increment;
    unsigned long limit;
    int i;
    CARD64 c64;

    while (argc>1) {
	if (argv[1][0] == '-') {
	    switch (argv[1][1]) {
	    case 'c':
		if (argc==2) usage();
		counter = atoi(argv[2]);
		argv += 2;
		argc -= 2;
		break;
	    case 'i':
		if (argc==2) usage();
		increment = atoi(argv[2]);
		argv += 2;
		argc -= 2;
		break;
	    case 'l':
		if (argc==2) usage();
		limit = atoi(argv[2]);
		argv += 2;
		argc -= 2;
		break;
	    default:
		fprintf(stderr, "Unknown option -%c\n", argv[1][1]);
		usage();
	    }
	} else {
	    usage();
	}
    }

    dpy = XOpenDisplay(NULL);

    if (!dpy) {
	fprintf(stderr, "Failed to open display!\n");
	exit(-1);
    }

    if (!XSyncQueryExtension(dpy, &sync_event, &sync_error)) {
	fprintf(stderr, "Failed to find SYNC extension\n");
	exit(-1);
    }

    if (counter == 0) {
	cvtLongToC64(c64, 1);
	counter = XSyncCreateCounter(dpy, c64);
    }
    printf("Counter is %u\n", counter);

    if (increment==0) {
	increment = 1;
    }
    if (limit==0) {
	limit = 30;
    }
    for (i=1; i<=limit; i+=increment) {
	cvtLongToC64(c64, increment);
	XSyncAdvance(dpy, counter, c64);
	XFlush(dpy);
	fprintf(stderr, ".");
	sleep(1);
	if (XPending(dpy)) {
	    fprintf(stderr, "*");
	}
    }

    fprintf(stderr, "\n");
}
