#include <stdio.h>
#include <math.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h>
#include "particle.h"
#include "setup.h"

#define TOKSEP		" ,\t\n"

static ProcList *ActionProcedures;
static SystemList *AllSystems = NULL;

XtResource systemResources[] = {
{"friction", "Friction", "Fixed", sizeof(int),
     XtOffset(System *, friction), XtRString, (caddr_t)"0"},
{"ddx", "DDX", "Fixed", sizeof(int),
     XtOffset(System *, ddx), XtRString, (caddr_t)"0"},
{"ddy", "DDY", "Fixed", sizeof(int),
     XtOffset(System *, ddy), XtRString, (caddr_t)"-1"},
{"x1", "X1", "Fixed", sizeof(int),
     XtOffset(System *, pos.x1), XtRString, (caddr_t)"0"},
{"y1", "Y1", "Fixed", sizeof(int),
     XtOffset(System *, pos.y1), XtRString, (caddr_t)"0"},
{"x2", "X2", "Fixed", sizeof(int),
     XtOffset(System *, pos.x2), XtRString, (caddr_t)"0"},
{"y2", "Y2", "Fixed", sizeof(int),
     XtOffset(System *, pos.y2), XtRString, (caddr_t)"0"},
{"dx1", "DX1", "Fixed", sizeof(int),
     XtOffset(System *, vel.x1), XtRString, (caddr_t)"0"},
{"dy1", "DY1", "Fixed", sizeof(int),
     XtOffset(System *, vel.y1), XtRString, (caddr_t)"0"},
{"dx2", "DX2", "Fixed", sizeof(int),
     XtOffset(System *, vel.x2), XtRString, (caddr_t)"0"},
{"dy2", "DY2", "Fixed", sizeof(int),
     XtOffset(System *, vel.y2), XtRString, (caddr_t)"0"},
{"r1", "R1", "Fixed", sizeof(int),
     XtOffset(System *, vel.r1), XtRString, (caddr_t)"0"},
{"theta1", "Theta1", "Fixed", sizeof(int),
     XtOffset(System *, vel.t1), XtRString, (caddr_t)"0"},
{"r2", "R2", "Fixed", sizeof(int),
     XtOffset(System *, vel.r2), XtRString, (caddr_t)"0"},
{"theta2", "Theta2", "Fixed", sizeof(int),
     XtOffset(System *, vel.t2), XtRString, (caddr_t)"0"},
{"timeMin", "TimeMin", XtRInt, sizeof(int),
     XtOffset(System *, time_min), XtRString, (caddr_t)"0"},
{"timeMax", "TimeMax", XtRInt, sizeof(int),
     XtOffset(System *, time_max), XtRString, (caddr_t)"0"},
{"bounce", "Bounce", XtRFloat, sizeof(float),
     XtOffset(System *, bounce), XtRString, (caddr_t)"0.10"},
{"startN", "StartN", XtRInt, sizeof(int),
     XtOffset(System *, start_n), XtRString, (caddr_t)"1"},
{"startD", "StartD", XtRInt, sizeof(int),
     XtOffset(System *, start_d), XtRString, (caddr_t)"1"},
{"startC", "StartC", XtRInt, sizeof(int),
     XtOffset(System *, start_c), XtRString, (caddr_t)"1"},
{"clipX1", "ClipX1", "Fixed", sizeof(int),
     XtOffset(System *, clip.x1), XtRString, (caddr_t)"0"},
{"clipY1", "ClipY1", "Fixed", sizeof(int),
     XtOffset(System *, clip.y1), XtRString, (caddr_t)"0"},
{"clipX2", "ClipX2", "Fixed", sizeof(int),
     XtOffset(System *, clip.x2), XtRString, (caddr_t)"128000"},
{"clipY2", "ClipY2", "Fixed", sizeof(int),
     XtOffset(System *, clip.y2), XtRString, (caddr_t)"128000"},
{"colour", "Colour", XtRPixel, sizeof(Pixel),
     XtOffset(System *, colour), XtRString,(caddr_t)"White"},
{"procedures", "Procedures", "ProcList", sizeof(ProcList),
     XtOffset(System *, procs), XtRString,
     (caddr_t)""},
{"newSystem", "NewSystem", "System", sizeof(System *),
     XtOffset(System *, new_system), XtRImmediate, NULL},
{"babySystem", "BabySystem", "System", sizeof(System *),
     XtOffset(System *, baby_system), XtRImmediate, NULL}
};

XtResource systemTypeResources[] = {
{"systems", "Systems", XtRString, sizeof(int),
     XtOffset(SystemType*, systems), XtRString, (caddr_t)"default"}
};

XtResource clipBoxResources[] = {
{"x1", "X1", "Fixed", sizeof(int),
     XtOffset(BoxRec *, x1), XtRString, (caddr_t)"0"},
{"x2", "X2", "Fixed", sizeof(int),
     XtOffset(BoxRec *, x2), XtRString, (caddr_t)"128000"},
{"y1", "Y1", "Fixed", sizeof(int),
     XtOffset(BoxRec *, y1), XtRString, (caddr_t)"0"},
{"y2", "Y2", "Fixed", sizeof(int),
     XtOffset(BoxRec *, y2), XtRString, (caddr_t)"128000"}
};

static Proc *FindProc(procs, name)
ProcList *procs;
char *name;
{
    procs = procs->next;

    while (procs && strcmp(name,procs->proc.name)!=0) {
	procs = procs->next;
    }

    if (procs) {
	return &procs->proc;
    } else {
	return NULL;
    } 
}

/*ARGSUSED*/
static Boolean CvtStringToProcList(dpy, args, num_args, from, to, env)
Display *dpy;
XrmValuePtr args;
Cardinal *num_args;
XrmValuePtr from, to;
XtPointer *env;
{
    char *ptr, *end;
    ProcList *procs = ActionProcedures;
    ProcList *res = CreateProcList();
    Proc *proc;
    char buffer[500];

    strcpy(buffer, (String)from->addr);
    end = buffer + strlen(buffer) - 1;
    ptr = buffer;

    while (ptr < end) {
	ptr = (char *)strtok(ptr, TOKSEP);
	if (!ptr) break;

	proc = FindProc(procs, ptr);

	if (proc==NULL) {
	    XtDisplayStringConversionWarning(dpy,
					     ptr,
					     "ProcList");
	    return False;
	}

	AddToProcList(res, proc->name, proc->proc);

	ptr = ptr + strlen(ptr) + 1;
    }

    if (to->addr != NULL) {
	if (to->size < sizeof(ProcList *)) {
	    to->size = sizeof(ProcList *);
	    return False;
	}
	*(ProcList **)(to->addr) = res;
    } else {
	static ProcList *static_val;
	static_val = res;

	to->addr = (XtPointer)&static_val;
    }
    to->size = sizeof(ProcList *);

    return True;
}

/*ARGSUSED*/
static Boolean CvtStringToFunction(dpy, args, num_args, from, to, env)
Display *dpy;
XrmValuePtr args;
Cardinal *num_args;
XrmValuePtr from, to;
XtPointer *env;
{
    String name = (String)from->addr;
    ProcList *procs = ActionProcedures;

    while (procs) {
	if (strcmp(name,procs->proc.name)==0) {
	    ProcList *value = procs;

	    if (to->addr != NULL) {
		if (to->size < sizeof(ProcList *)) {
		    to->size = sizeof(ProcList *);
		    return False;
		}
		*(ProcList **)(to->addr) = value;
	    } else {
		static ProcList *static_val;
		static_val = value;

		to->addr = (XtPointer)&static_val;
	    }
	    to->size = sizeof(ProcList *);
	    return True;
	}
	procs = procs->next;
    }

    XtDisplayStringConversionWarning(dpy, name, "Function");

    return False;
}

/*ARGSUSED*/
static Boolean CvtStringToSystem(dpy, args, num_args, from, to, env)
Display *dpy;
XrmValuePtr args;
Cardinal *num_args;
XrmValuePtr from, to;
XtPointer *env;
{
    String name = (String)from->addr;
    SystemList *systems = AllSystems;

    while (systems) {
	if (strcmp(name,systems->name)==0) {
	    System *value = systems->system;

	    if (to->addr != NULL) {
		if (to->size < sizeof(System  *)) {
		    to->size = sizeof(System  *);
		    return False;
		}
		*(System  **)(to->addr) = value;
	    } else {
		static System  *static_val;
		static_val = value;

		to->addr = (XtPointer)&static_val;
	    }
	    to->size = sizeof(System  *);
	    return True;
	}
	systems = systems->next;
    }

    XtDisplayStringConversionWarning(dpy, name, "System");

    return False;
}

/*ARGSUSED*/
static Boolean CvtStringToFixed(dpy, args, num_args, from, to, env)
Display *dpy;
XrmValuePtr args;
Cardinal *num_args;
XrmValuePtr from, to;
XtPointer *env;
{
    String name = (String)from->addr;
    double fval = atof(name);
    int value = (int)(fval*(FRAC(1)));

    if (to->addr != NULL) {
	if (to->size < sizeof(int)) {
	    to->size = sizeof(int);
	    return False;
	}
	*(int *)(to->addr) = value;
    } else {
	static int static_val;
	static_val = value;

	to->addr = (XtPointer)&static_val;
    }
    to->size = sizeof(int);
    return True;
}

static System *CreateSystem(w, name, nextSys)
Widget w;
char *name;
System *nextSys;
{
    System *sys = (System *)malloc(sizeof(System));
    SystemList *syslist = (SystemList *)malloc(sizeof(SystemList));

    XtVaCreateManagedWidget(name, labelWidgetClass,
			    w,
			    NULL);

    XtVaGetSubresources(w, (XtPointer)sys, name, "System",
			systemResources,
			XtNumber(systemResources),
			NULL);

#ifdef XXX
    sys->initialSpace = space;
#endif
    sys->next = nextSys;
    sys->babies = NULL;
    sys->adults = NULL;
    sys->corpses = NULL;

    sys->gc = XCreateGC(XtDisplay(w), DefaultRootWindow(XtDisplay(w)),
			0, NULL);
    XSetForeground(XtDisplay(w), sys->gc, sys->colour);

    sys->fore_points = sys->points1;
    sys->back_points = sys->points2;
    sys->n_back = 0;

    strcpy(syslist->name, name);
    syslist->system = sys;
    syslist->next = AllSystems;
    AllSystems = syslist;

    return sys;
}

static System *CreateType(top, pane, name, nextSys)
Widget top, pane;
char *name;
System *nextSys;
{
    SystemType *type = (SystemType *)malloc(sizeof(SystemType));
    System *sys = nextSys;
    Widget b;
    String ptr, end;
    char buffer[100];

    b = XtVaCreateManagedWidget(name, boxWidgetClass,
				pane,
				NULL);
    XtVaCreateManagedWidget(name,commandWidgetClass,
			    b,
			    NULL);

    XtVaGetSubresources(top, (XtPointer)type, name, "SystemType",
			systemTypeResources,
			XtNumber(systemTypeResources),
			NULL);

    strcpy(buffer, type->systems);
    end = buffer + strlen(buffer) - 1;
    ptr = buffer;
    while (ptr < end) {
	ptr = (char *)strtok(ptr, TOKSEP);
	if (!ptr) break;
	sys = CreateSystem(b, ptr, sys);

	ptr = ptr + strlen(ptr) + 1;
    }

    return sys;
}

void InitSetup(appC)
XtAppContext appC;
{
    ActionProcedures = CreateProcList();
    AddParticleProcs(ActionProcedures);

    XtAppSetTypeConverter(appC, XtRString, "ProcList",
			  CvtStringToProcList, NULL, 0,
			  XtCacheAll, NULL);
    XtAppSetTypeConverter(appC, XtRString, "System",
			  CvtStringToSystem, NULL, 0,
			  XtCacheAll, NULL);
    XtAppSetTypeConverter(appC, XtRString, "Fixed",
			  CvtStringToFixed, NULL, 0,
			  XtCacheAll, NULL);
}

Animation *DoSetup(top, pane, pic, system_types)
Widget top, pane, pic;
char *system_types;
{
    System *type = NULL;
    String ptr, end;
    char buffer[100];

    strcpy(buffer, system_types);
    end = buffer + strlen(buffer) - 1;
    ptr = buffer;
    while (ptr < end) {
	ptr = (char *)strtok(ptr, TOKSEP);
	if (!ptr) break;

	type = CreateType(top, pane, ptr, type);

	ptr = ptr + strlen(ptr) + 1;
    };

    return InitAnimation(type);
}
