/***********************************************************
Copyright 1991 by Digital Equipment Corporation, Maynard, Massachusetts,
the Massachusetts Institute of Technology, Cambridge, Massachusetts,
and Olivetti Research Limited, Cambridge, England.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital, Olivetti or MIT
not be used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL AND OLIVETTI DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT
SHALL THEY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

******************************************************************/

#include "sync.h"

#ifndef _SYNC_OS

/*
 * Sync Info
 */
typedef struct _xSyncInfoReq {
    CARD8	reqType;
    CARD8	syncReqType;
    CARD16	length B16;
} xSyncInfoReq;
#define sz_xSyncInfoReq		4

typedef struct {
    BYTE	type;
    CARD8	unused;
    CARD16	sequenceNumber B16;
    CARD32	length B32;
    CARD8	majorVersion;
    CARD8	minorVersion;
    CARD16	pad1 B16;
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
    CARD32	nCounters;		/* followed by system counters */
} xSyncInfoReply;
#define sz_xSyncInfoReply	32

/*
 * Create Counter
 */
typedef struct _xSyncCreateCounterReq {
    CARD8	reqType;
    CARD8	syncReqType;
    CARD16	length B16;
    CARD32	cid B32;
    CARD32	initial_value_lo B32;
    CARD32      initial_value_hi B32;
} xSyncCreateCounterReq;
#define sz_xSyncCreateCounterReq	16

/*
 * Destroy Counter
 */
typedef struct _xSyncDestroyCounterReq {
    CARD8	reqType;
    CARD8	syncReqType;
    CARD16	length B16;
    CARD32	counter B32;
} xSyncDestroyCounterReq;
#define sz_xSyncDestroyCounterReq	8

/*
 * Query Counter
 */
typedef struct _xSyncQueryCounterReq {
    CARD8	reqType;
    CARD8	syncReqType;
    CARD16	length B16;
    CARD32	counter B32;
} xSyncQueryCounterReq;
#define sz_xSyncQueryCounterReq		8


typedef struct {
    BYTE	type;
    CARD8	unused;
    CARD16	sequenceNumber B16;
    CARD32	length B32;
    CARD32	value_lo B32;
    CARD32	value_hi B32;
    CARD32	pad0 B32;
    CARD32	pad1 B32;
    CARD32	pad2 B32;
    CARD32	pad3 B32;
} xSyncQueryCounterReply;
#define sz_xSyncQueryCounterReply	32

/*
 * Await
 */
typedef struct _xSyncAwaitReq {
    CARD8	reqType;
    CARD8	syncReqType;
    CARD16	length B16;
} xSyncAwaitReq;
#define sz_xSyncAwaitReq		4

/*
 * Advance
 */
typedef struct _xSyncAdvanceReq {
    CARD8	reqType;
    CARD8	syncReqType;
    CARD16	length B16;
    CARD32	counter B32;
    CARD32	amount_lo B32;
    CARD32	amount_hi B32;
} xSyncAdvanceReq;
#define sz_xSyncAdvanceReq		16

/*
 * Create Alarm
 */
typedef struct _xSyncCreateAlarmReq {
    CARD8	reqType;
    CARD8	syncReqType;
    CARD16	length B16;
    CARD32	id B32;
    CARD32      valueMask;
} xSyncCreateAlarmReq;
#define sz_xSyncCreateAlarmReq		12

/*
 * Destroy Alarm
 */
typedef struct _xSyncDestroyAlarmReq {
    CARD8	reqType;
    CARD8	syncReqType;
    CARD16	length B16;
    CARD32	alarm B32;
} xSyncDestroyAlarmReq;
#define sz_xSyncDestroyAlarmReq		8

/*
 * Query Alarm
 */
typedef struct _xSyncQueryAlarmReq {
    CARD8	reqType;
    CARD8	syncReqType;
    CARD16	length B16;
    CARD32	alarm B32;
} xSyncQueryAlarmReq;
#define sz_xSyncQueryAlarmReq		8

typedef struct {
    BYTE	type;
    CARD8	unused;
    CARD16	sequenceNumber B16;
    CARD32	length B32;
    CARD32	counter B32;
    CARD32	value_lo B32;
    CARD32	value_hi B32;
    CARD32	increment_lo B32;
    CARD32	increment_hi B32;
    CARD32	limit_lo B32;
    CARD32	limit_hi B32;
    BYTE        events;
    BYTE        state;
    BYTE	pad0;
    BYTE	pad1;
} xSyncQueryAlarmReply;
#define sz_xSyncQueryAlarmReply		40

/*
 * Change Alarm
 */
typedef struct _xSyncChangeAlarmReq {
    CARD8	reqType;
    CARD8	syncReqType;
    CARD16	length B16;
    CARD32	alarm B32;
    CARD32	valueMask B32;
} xSyncChangeAlarmReq;
#define sz_xSyncChangeAlarmReq		12

/*
 * Events
 */
typedef struct _xAdvanceNotifyEvent {
    BYTE	type;
    BYTE	kind;
    CARD16	sequenceNumber B16;
    CARD32	counter B32;
    CARD32	wait_value_lo B32;
    CARD32	wait_value_hi B32;
    CARD32	counter_value_lo B32;
    CARD32	counter_value_hi B32;
    CARD32	time B32;
    CARD32      pad0 B32;
} xAdvanceNotifyEvent;

typedef struct _xAlarmNotifyEvent {
    BYTE	type;
    BYTE	kind;
    CARD16	sequenceNumber B16;
    CARD32	alarm B32;
    CARD32	counter_value_lo B32;
    CARD32	counter_value_hi B32;
    CARD32	alarm_value_lo B32;
    CARD32	alarm_value_hi B32;
    CARD32	time B32;
    BYTE        state;
    BYTE        pad0;
    BYTE        pad1;
    BYTE        pad2;
} xAlarmNotifyEvent;

#endif /* _SYNC_OS */

#ifdef _SYNC_SERVER

/*
 * The System Counter interface
 * XXX: This is rather broken at the moment
 */
extern pointer SyncCreateSystemCounter(
#if NeedFunctionPrototypes
    Atom		/* name */,
    CARD64  	/* inital_value */,
    CARD64  	/* resolution */,
    void	(*query)(),
    void	(*newwait)()
#endif
);

extern void SyncChangeCounter(
#if NeedFunctionPrototypes
    pointer		/* pCounter*/,
    CARD64  		/* new_value */
#endif
);

extern void SyncAdvanceCounter(
#if NeedFunctionPrototypes
    pointer		/* pCounter*/,
    CARD64  		/* amount */
#endif
);

extern void InitServertime();

#endif /* _SYNC_SERVER */
